@@ -26,6 +26,9 @@
 #ifndef CGAL_MESH_COMPLEX_3_IN_TRIANGULATION_3_H
 #define CGAL_MESH_COMPLEX_3_IN_TRIANGULATION_3_H
 
+#include <CGAL/license/Mesh_3.h>
+
+
 
 #include <CGAL/iterator.h>
 
@@ -549,7 +552,7 @@ private:
   void add_to_complex(const Internal_edge& edge, const Curve_segment_index& index)
   {
     CGAL_precondition(!is_in_complex(edge));
-#ifdef CGAL_MESH_3_PROTECTION_DEBUG
+#if CGAL_MESH_3_PROTECTION_DEBUG & 1
     std::cerr << "Add edge ( " << edge.left->point()
               << " , " << edge.right->point() << " ), curve_index=" << index
               << " to c3t3.\n";
@@ -703,14 +706,15 @@ is_valid(bool verbose) const
   for ( typename Edge_map::const_iterator it = edges_.begin(),
        end = edges_.end() ; it != end ; ++it )
   {
-    const Bare_point& p = it->right->point().point();
-    const Bare_point& q = it->left->point().point();
-
     typename Tr::Geom_traits::Construct_sphere_3 sphere =
       this->triangulation().geom_traits().construct_sphere_3_object();
-
     typename Tr::Geom_traits::Do_intersect_3 do_intersect =
       this->triangulation().geom_traits().do_intersect_3_object();
+    typename Tr::Geom_traits::Construct_point_3 wp2p =
+      this->triangulation().geom_traits().construct_point_3_object();
+
+    const Bare_point& p = wp2p(it->right->point());
+    const Bare_point& q = wp2p(it->left->point());
 
     const FT& sq_rp = it->right->point().weight();
     const FT& sq_rq = it->left->point().weight();
