@@ -22,10 +22,14 @@
 #ifndef CGAL_POLYGON_MESH_PROCESSING_ORIENT_POLYGON_SOUP
 #define CGAL_POLYGON_MESH_PROCESSING_ORIENT_POLYGON_SOUP
 
+#include <CGAL/license/Polygon_mesh_processing/orientation.h>
+
+
 #include <CGAL/tuple.h>
 #include <CGAL/array.h>
 #include <CGAL/assertions.h>
 #include <boost/foreach.hpp>
+#include <boost/container/flat_set.hpp>
 
 #include <set>
 #include <map>
@@ -40,9 +44,11 @@ namespace Polygon_mesh_processing {
 
 namespace internal {
 
-template<class Point_3, class Polygon_3>
-class Polygon_soup_orienter
+template<class PointRange, class PolygonRange>
+struct Polygon_soup_orienter
 {
+  typedef typename PointRange::value_type                               Point_3;
+  typedef typename PolygonRange::value_type                           Polygon_3;
 /// Index types
   typedef typename std::iterator_traits<
             typename Polygon_3::iterator >::value_type                     V_ID;
@@ -50,9 +56,9 @@ class Polygon_soup_orienter
 //  typedef int                                                             CC_ID;
   typedef std::pair<V_ID, V_ID>                                       V_ID_pair;
 /// Container types
-  typedef std::vector<Point_3>                                           Points;
-  typedef std::vector<Polygon_3>                                       Polygons;
-  typedef std::map<V_ID_pair, std::set<P_ID> >                         Edge_map;
+  typedef PointRange                                                     Points;
+  typedef PolygonRange                                                 Polygons;
+  typedef std::map<V_ID_pair, boost::container::flat_set<P_ID> >       Edge_map;
   typedef typename Edge_map::iterator                         Edge_map_iterator;
   typedef std::set<V_ID_pair>                                      Marked_edges;
 
@@ -66,27 +72,25 @@ class Polygon_soup_orienter
   /// for each polygon referenced by its position in `polygons`, indicates
   /// the connected component it belongs too after orientation.
 //  std::vector< CC_ID > polygon_cc_id;
-  /// for each vertex, indicates the list of polygon containing it
-  std::vector< std::vector<P_ID> > incident_polygons_per_vertex;
 
 /// Utility functions
-  V_ID_pair canonical_edge(V_ID i, V_ID j)
+  static V_ID_pair canonical_edge(V_ID i, V_ID j)
   {
     return i<j ? V_ID_pair(i,j):V_ID_pair(j,i);
   }
 
-  bool is_edge_marked(V_ID i, V_ID j)
+  static bool is_edge_marked(V_ID i, V_ID j, Marked_edges& marked_edges)
   {
     return marked_edges.count(canonical_edge(i,j)) > 0;
   }
 
-  void set_edge_marked(V_ID i, V_ID j)
+  static void set_edge_marked(V_ID i, V_ID j, Marked_edges& marked_edges)
   {
     marked_edges.insert(canonical_edge(i,j));
   }
 
-  cpp11::array<V_ID,3>
-  get_neighbor_vertices(V_ID v_id, P_ID polygon_index)
+  static cpp11::array<V_ID,3>
+  get_neighbor_vertices(V_ID v_id, P_ID polygon_index, const Polygons& polygons)
   {
     std::size_t nbv = polygons[polygon_index].size(), pvid=0;
     for (; pvid!=nbv; ++pvid)
@@ -97,28 +101,28 @@ class Polygon_soup_orienter
     return make_array(prev,v_id,next);
   }
 
-  std::pair<V_ID,P_ID>
-  next_cw_vertex_around_source(V_ID src, V_ID tgt)
+  static std::pair<V_ID,P_ID>
+  next_cw_vertex_around_source(V_ID src, V_ID tgt, const Polygons& polygons, Edge_map& edges, Marked_edges& marked_edges)
   {
     typedef std::pair<V_ID,P_ID> VID_and_PID;
-    if ( is_edge_marked(src,tgt) ) return VID_and_PID(src,300612);
+    if ( is_edge_marked(src,tgt,marked_edges) ) return VID_and_PID(src,300612);
     Edge_map_iterator em_it=edges.find(V_ID_pair(tgt, src));
     if ( em_it==edges.end() ) return VID_and_PID(src,300612);// the vertex is on the border
     CGAL_assertion(em_it->second.size()==1);
     P_ID p_id = *(em_it->second.begin());
-    return VID_and_PID(get_neighbor_vertices(src, p_id)[2], p_id);
+    return VID_and_PID(get_neighbor_vertices(src, p_id, polygons)[2], p_id);
   }
 
-  std::pair<V_ID,P_ID>
-  next_ccw_vertex_around_target(V_ID src, V_ID tgt)
+  static std::pair<V_ID,P_ID>
+  next_ccw_vertex_around_target(V_ID src, V_ID tgt, const Polygons& polygons, Edge_map& edges, Marked_edges& marked_edges)
   {
     typedef std::pair<V_ID,P_ID> VID_and_PID;
-    if ( is_edge_marked(src,tgt) ) return VID_and_PID(tgt,300612);
+    if ( is_edge_marked(src,tgt,marked_edges) ) return VID_and_PID(tgt,300612);
     Edge_map_iterator em_it=edges.find(V_ID_pair(tgt, src));
     if ( em_it==edges.end() ) return VID_and_PID(tgt,300612);// the vertex is on the border
     CGAL_assertion(em_it->second.size()==1);
     P_ID p_id = *(em_it->second.begin());
-    return VID_and_PID(get_neighbor_vertices(tgt, p_id)[0], p_id);
+    return VID_and_PID(get_neighbor_vertices(tgt, p_id, polygons)[0], p_id);
   }
 
   void inverse_orientation(const std::size_t index) {
@@ -136,43 +140,10 @@ class Polygon_soup_orienter
   }
 
 /// Functions filling containers
-  void fill_edge_map() {
-    // Fill edges
-    edges.clear();
-    for(P_ID i = 0; i < polygons.size(); ++i)
-    {
-      const P_ID size = polygons[i].size();
-      for(P_ID j = 0; j < size; ++j) {
-        V_ID i0 = polygons[i][j];
-        V_ID i1 = polygons[i][ (j+1)%size];
-        edges[V_ID_pair(i0, i1)].insert(i);
-      }
-    }
-
-    // Fill non-manifold edges
-    marked_edges.clear();
-    for(P_ID i = 0; i < polygons.size(); ++i)
-    {
-      const P_ID size = polygons[i].size();
-      for(P_ID j = 0; j < size; ++j) {
-        V_ID i0 = polygons[i][j];
-        V_ID i1 = polygons[i][ (j+1)%size ];
-
-        std::size_t nb_edges = 0;
-        Edge_map_iterator em_it = edges.find( V_ID_pair(i0, i1) );
-        if ( em_it!=edges.end() ) nb_edges += em_it->second.size();
-        em_it = edges.find( V_ID_pair(i1, i0) );
-        if ( em_it!=edges.end() ) nb_edges += em_it->second.size();
-
-        if( nb_edges > 2 ) set_edge_marked(i0,i1);
-      }
-    }
-  }
-
-  void fill_incident_polygons_per_vertex()
+  static void fill_incident_polygons_per_vertex(
+    const Polygons& polygons,
+    std::vector< std::vector<P_ID> >& incident_polygons_per_vertex)
   {
-    incident_polygons_per_vertex.resize(points.size());
-
     P_ID nb_polygons=polygons.size();
     for(P_ID ip=0; ip<nb_polygons; ++ip)
     {
@@ -181,12 +152,47 @@ class Polygon_soup_orienter
     }
   }
 
-public:
-
   Polygon_soup_orienter(Points& points, Polygons& polygons)
     : points(points), polygons(polygons)
+  {}
+
+//filling containers
+  static void fill_edge_map(Edge_map& edges, Marked_edges& marked_edges, const Polygons& polygons) {
+    // Fill edges
+    edges.clear();
+    for (P_ID i = 0; i < polygons.size(); ++i)
+    {
+      const P_ID size = polygons[i].size();
+      for (P_ID j = 0; j < size; ++j) {
+        V_ID i0 = polygons[i][j];
+        V_ID i1 = polygons[i][(j + 1) % size];
+        edges[V_ID_pair(i0, i1)].insert(i);
+      }
+    }
+
+    // Fill non-manifold edges
+    marked_edges.clear();
+    for (P_ID i = 0; i < polygons.size(); ++i)
+    {
+      const P_ID size = polygons[i].size();
+      for (P_ID j = 0; j < size; ++j) {
+        V_ID i0 = polygons[i][j];
+        V_ID i1 = polygons[i][(j + 1) % size];
+
+        std::size_t nb_edges = 0;
+        Edge_map_iterator em_it = edges.find(V_ID_pair(i0, i1));
+        if (em_it != edges.end()) nb_edges += em_it->second.size();
+        em_it = edges.find(V_ID_pair(i1, i0));
+        if (em_it != edges.end()) nb_edges += em_it->second.size();
+
+        if (nb_edges > 2) set_edge_marked(i0, i1, marked_edges);
+      }
+    }
+  }
+
+  void fill_edge_map()
   {
-    fill_edge_map();
+    fill_edge_map(edges, marked_edges, polygons);
   }
 
   /// We try to orient polygon consistently by walking in the dual graph, from
@@ -238,7 +244,7 @@ public:
           const V_ID i1 = polygons[to_be_oriented_index][ih];
           const V_ID i2 = polygons[to_be_oriented_index][ihp1];
 
-          if( is_edge_marked(i1,i2) ) continue;
+          if( is_edge_marked(i1,i2,marked_edges) ) continue;
 
           // edge (i1,i2)
           Edge_map_iterator it_same_orient = edges.find(V_ID_pair(i1, i2));
@@ -259,7 +265,7 @@ public:
             if(oriented[index])
             {
               // polygon already oriented but its orientation is not compatible ---> mark the edge and continue
-              set_edge_marked(i1,i2);
+              set_edge_marked(i1,i2,marked_edges);
               continue; // next edge
             }
 
@@ -309,7 +315,9 @@ public:
   /// each but one set of incident polygons.
   void duplicate_singular_vertices()
   {
-    fill_incident_polygons_per_vertex();
+    // for each vertex, indicates the list of polygon containing it
+    std::vector< std::vector<P_ID> > incident_polygons_per_vertex(points.size());
+    fill_incident_polygons_per_vertex(polygons, incident_polygons_per_vertex);
     std::vector< std::pair<V_ID, std::vector<P_ID> > > vertices_to_duplicate;
 
     V_ID nbv = static_cast<V_ID>( points.size() );
@@ -331,7 +339,7 @@ public:
           vertices_to_duplicate.back().first=v_id;
         }
 
-        const cpp11::array<V_ID,3>& neighbors = get_neighbor_vertices(v_id,p_id);
+        const cpp11::array<V_ID,3>& neighbors = get_neighbor_vertices(v_id,p_id,polygons);
 
         V_ID next = neighbors[2];
 
@@ -340,7 +348,7 @@ public:
 
         do{
           P_ID other_p_id;
-          cpp11::tie(next, other_p_id) = next_cw_vertex_around_source(v_id, next);
+          cpp11::tie(next, other_p_id) = next_cw_vertex_around_source(v_id, next, polygons, edges, marked_edges);
           if (next==v_id) break;
           visited_polygons.insert(other_p_id);
           if( !first_pass)
@@ -353,7 +361,7 @@ public:
           next = neighbors[0];
           do{
             P_ID other_p_id;
-            cpp11::tie(next, other_p_id) = next_ccw_vertex_around_target(next, v_id);
+            cpp11::tie(next, other_p_id) = next_ccw_vertex_around_target(next, v_id, polygons, edges, marked_edges);
             if (next==v_id) break;
             visited_polygons.insert(other_p_id);
             if( !first_pass)
@@ -375,11 +383,66 @@ public:
         replace_vertex_index_in_polygon(polygon_id, vid_and_pids.first, new_index);
     }
   }
+
+  static bool has_singular_vertices(
+    std::size_t nb_points,
+    const Polygons& polygons,
+    Edge_map& edges,
+    Marked_edges& marked_edges)
+  {
+    // for each vertex, indicates the list of polygon containing it
+    std::vector< std::vector<P_ID> > incident_polygons_per_vertex(nb_points);
+    fill_incident_polygons_per_vertex(polygons, incident_polygons_per_vertex);
+
+    V_ID nbv = static_cast<V_ID>( nb_points );
+    for (V_ID v_id = 0; v_id < nbv; ++v_id)
+    {
+      const std::vector< P_ID >& incident_polygons = incident_polygons_per_vertex[v_id];
+
+      if ( incident_polygons.empty() ) continue; //isolated vertex
+      std::set<P_ID> visited_polygons;
+
+      bool first_pass = true;
+      BOOST_FOREACH(P_ID p_id, incident_polygons)
+      {
+        if ( !visited_polygons.insert(p_id).second ) continue; // already visited
+
+        if (!first_pass)
+          return false; //there will be duplicate vertices
+
+        const cpp11::array<V_ID,3>& neighbors = get_neighbor_vertices(v_id,p_id,polygons);
+
+        V_ID next = neighbors[2];
+
+        do{
+          P_ID other_p_id;
+          cpp11::tie(next, other_p_id) = next_cw_vertex_around_source(v_id, next, polygons, edges, marked_edges);
+          if (next==v_id) break;
+          visited_polygons.insert(other_p_id);
+        }
+        while(next!=neighbors[0]);
+
+        if (next==v_id){
+          /// turn the otherway round
+          next = neighbors[0];
+          do{
+            P_ID other_p_id;
+            cpp11::tie(next, other_p_id) = next_ccw_vertex_around_target(next, v_id, polygons, edges, marked_edges);
+            if (next==v_id) break;
+            visited_polygons.insert(other_p_id);
+          }
+          while(true);
+        }
+        first_pass=false;
+      }
+    }
+    return true;
+  }
 };
 } // namespace internal
 
 /**
- * \ingroup PMP_repairing_grp
+ * \ingroup PMP_orientation_grp
  * tries to consistently orient a soup of polygons in 3D space.
  * When it is not possible to produce a combinatorial manifold surface,
  * some points are duplicated.
@@ -397,9 +460,11 @@ public:
  *
  * The algorithm is described in \cgalCite{gueziec2001cutting}.
  *
- * @tparam Point the point type
- * @tparam Polygon a `std::vector<std::size_t>` containing the indices
- *         of the points of the face
+ * @tparam PointRange a model of the concepts `RandomAccessContainer`
+ * and `BackInsertionSequence` whose value type is the point type
+ * @tparam PolygonRange a model of the concept `RandomAccessContainer`
+ * whose value_type is a model of the concept `RandomAccessContainer`
+ * whose value_type is `std::size_t`.
  *
  * @param points points of the soup of polygons. Some points might be pushed back to resolve
  *               non-manifold or non-orientability issues.
@@ -409,12 +474,13 @@ public:
  * @return `false` if some points were duplicated, thus producing a self-intersecting polyhedron.
  *
  */
-template <class Point, class Polygon>
-bool orient_polygon_soup(std::vector<Point>& points,
-                         std::vector<Polygon>& polygons)
+template <class PointRange, class PolygonRange>
+bool orient_polygon_soup(PointRange& points,
+                         PolygonRange& polygons)
 {
   std::size_t inital_nb_pts = points.size();
-  internal::Polygon_soup_orienter<Point, Polygon> orienter(points, polygons);
+  internal::Polygon_soup_orienter<PointRange, PolygonRange> orienter(points, polygons);
+  orienter.fill_edge_map();
   orienter.orient();
   orienter.duplicate_singular_vertices();
 
