@@ -19,6 +19,9 @@
 #ifndef CGAL_RIDGE_3_H_
 #define CGAL_RIDGE_3_H_
 
+#include <CGAL/license/Ridges_3.h>
+
+
 #include <utility>
 #include <list>
 #include <map>
@@ -27,9 +30,10 @@
 #include <CGAL/Min_sphere_d.h>
 #include <CGAL/Optimisation_d_traits_3.h>
 #include <CGAL/barycenter.h>
-#include <CGAL/property_map.h>
+#include <CGAL/boost/graph/properties.h>
 #include <CGAL/assertions.h>
 #include <boost/type_traits/is_same.hpp>
+#include <boost/foreach.hpp>
 
 namespace CGAL {
  
