@@ -14,7 +14,7 @@
 //
 // $URL$
 // $Id$
-// 
+//
 //
 // Author(s)     : Nico Kruithof <Nico@cs.rug.nl>
 
@@ -23,17 +23,16 @@
 
 #include <CGAL/license/Skin_surface_3.h>
 
-
 #include <CGAL/Skin_surface_refinement_policy_3.h>
 #include <CGAL/Polyhedron_3.h>
 
 namespace CGAL {
 
 template <class UnionOfBalls_3, class Polyhedron_3>
-void subdivide_union_of_balls_mesh_3(
-          const UnionOfBalls_3 &skin,
-          Polyhedron_3 &p, 
-          int nSubdiv = 1) {
+void subdivide_union_of_balls_mesh_3(const UnionOfBalls_3 &skin,
+                                     Polyhedron_3 &p,
+                                     int nSubdiv = 1)
+{
   while (nSubdiv > 0) {
     skin.subdivide_mesh_3(p);
     nSubdiv--;
